/*
 * Copyright (c) 1995, the EUROPAGATE consortium (see below).
 *
 * The EUROPAGATE consortium members are:
 *
 *    University College Dublin
 *    Danmarks Teknologiske Videnscenter
 *    An Chomhairle Leabharlanna
 *    Consejo Superior de Investigaciones Cientificas
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation, in whole or in part, for any purpose, is hereby granted,
 * provided that:
 *
 * 1. This copyright and permission notice appear in all copies of the
 * software and its documentation. Notices of copyright or attribution
 * which appear at the beginning of any file must remain unchanged.
 *
 * 2. The names of EUROPAGATE or the project partners may not be used to
 * endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 3. Users of this software (implementors and gateway operators) agree to
 * inform the EUROPAGATE consortium of their use of the software. This
 * information will be used to evaluate the EUROPAGATE project and the
 * software, and to plan further developments. The consortium may use
 * the information in later publications.
 * 
 * 4. Users of this software agree to make their best efforts, when
 * documenting their use of the software, to acknowledge the EUROPAGATE
 * consortium, and the role played by the software in their work.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED, OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL THE EUROPAGATE CONSORTIUM OR ITS MEMBERS BE LIABLE
 * FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF
 * ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND
 * ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/*
 * Iso2709 record management
 *
 * Europagate, 1994-1995.
 *
 * $Log: iso2709dump.c,v $
 * Revision 1.11  1995/05/16 09:40:54  adam
 * LICENSE.
 *
 * Revision 1.10  1995/03/31  09:43:24  adam
 * Removed test display.
 *
 * Revision 1.9  1995/03/30  14:22:18  adam
 * More work on new MARC anchor functions.
 *
 * Revision 1.8  1995/03/30  07:33:37  adam
 * New 2709 function: iso2709_mk.
 * First implementation of iso2709_a_insert.
 *
 * Revision 1.7  1995/03/28  16:07:07  adam
 * New function: iso2709_out. This function is the reverse of iso2709_cvt.
 *
 * Revision 1.6  1995/03/27  12:52:18  adam
 * A little more verbose in marc dump.
 *
 * Revision 1.5  1995/02/22  21:32:36  adam
 * Changed header.
 *
 * Revision 1.3  1995/02/10  17:05:18  adam
 * New function iso2709_display to display MARC records in a
 * line-by-line format. The iso2709_cvt function no longer
 * prints the record to stderr.
 *
 * Revision 1.2  1995/02/10  16:50:33  adam
 * Indicator field moved to 'struct iso2709_dir' from 'struct
 * iso2709_field'.
 * Function iso2709_rm implemented - to delete a MARC record.
 *
 * Revision 1.1.1.1  1995/02/09  17:27:11  adam
 * Initial version of email gateway under CVS control.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <iso2709.h>

static char *prog;

static Iso2709Rec copy_rec (Iso2709Rec rec_in)
{
    Iso2709Rec rec_out;
    Iso2709Anchor a_in, a_out;
    char *tag, *indicator, *identifier, *data;

    rec_out = iso2709_mk();

    a_in = iso2709_a_mk (rec_in);
    a_out = iso2709_a_mk (rec_out);

    do {
        if (!iso2709_a_info_field (a_in, &tag, &indicator, &identifier, &data))
	    break;
        iso2709_a_insert (a_out, tag, indicator, identifier, data);
    } while (iso2709_a_next (a_in));
    iso2709_a_rm (a_in);
    iso2709_a_rm (a_out);
    return rec_out;
}

int main (int argc, char **argv)
{
    char *buf;
    int no = 0;
    int start_pos = 0;
    int end_pos = 99999;
    char *output_file = NULL;
    char *input_file = NULL;
    char *filter = NULL;
    int verbose = 0;
    int quiet = 0;
    FILE *outf = NULL;
    FILE *inf = stdin;

    prog = *argv;
    while (--argc > 0)
    {
        if (**++argv == '-')
        {
            switch (argv[0][1])
            {
            case 'h':
            case 'H':
                fprintf (stderr, "iso2709dump [options] [file]\n");
                fprintf (stderr, "If no file is specified stdin is used\n");
                fprintf (stderr, "Options:\n");
                fprintf (stderr, " -p no        Starting position\n");
                fprintf (stderr, " -l end       Ending position\n");
                fprintf (stderr, " -o file      Raw MARC output file\n");
                fprintf (stderr, " -v           Verbose\n");
                fprintf (stderr, " -q           Quiet mode\n");
                exit (1);
            case 'p':
                if (argv[0][2])
                    start_pos = atoi (argv[0]+2);
                else if (argc > 0)
                {
                    --argc;
                    start_pos = atoi (*++argv);
                }
                else
                {
                    fprintf (stderr, "%s: missing start pos\n", prog);
                    exit (1);
                }
                break;
            case 'l':
                if (argv[0][2])
                    end_pos = atoi (argv[0]+2);
                else if (argc > 0)
                {
                    --argc;
                    end_pos = atoi (*++argv);
                }
                else
                {
                    fprintf (stderr, "%s: missing end pos\n", prog);
                    exit (1);
                }
                break;
            case 'o':
                if (argv[0][2])
                    output_file = argv[0]+2;
                else if (argc > 0)
                {
                    --argc;
                    output_file = *++argv;
                }
                else
                {
                    fprintf (stderr, "%s: missing output file\n", prog);
                    exit (1);
                }
                break;
            case 'v':
                verbose = 1;
                break;
            case 'q':
                quiet = 1;
                break;
	    case 'f':
                if (argv[0][2])
                    filter = argv[0]+2;
                else if (argc > 0)
                {
                    --argc;
                    filter = *++argv;
                }
                else
                {
                    fprintf (stderr, "%s: missing filter\n", prog);
                    exit (1);
                }
                break;
            default:
                fprintf (stderr, "%s: unknown option %s; use -h for help\n",
                         prog, *argv);
                exit (1);
            }
        }
        else
            input_file = *argv;
    }
    if (input_file)
    {
        inf = fopen (input_file, "r");
        if (!inf)
        {
            fprintf (stderr, "%s: cannot open %s: %s\n",
                     prog, input_file, strerror (errno));
            exit (1);
        }
    }
    if (output_file)
    {
        outf = fopen (output_file, "w");
        if (!outf)
        {
            fprintf (stderr, "%s: cannot open %s: %s\n",
                     prog, output_file, strerror (errno));
            exit (1);
        }
    }
    while ((buf = iso2709_read (inf)))
    {
        if (no >= start_pos && no <= end_pos)
        {
            char *obuf;
            int olen;
            Iso2709Rec rec_input;
            Iso2709Rec rec_output;
            
            rec_input = iso2709_cvt (buf);
            if (!quiet)
                printf ("------- %d --------\n", no);
            if (!rec_input)
            {
                if (!quiet)
                    printf ("Bad record\n");
                fprintf (stderr, "%s: bad record at position %d\n", prog, no);
                break;
            }
            olen = iso2709_out (rec_input, &obuf, 0);
	    rec_output = copy_rec (rec_input);
            if (!quiet)
                iso2709_display (rec_input, stdout);
            if (outf && fwrite (obuf, 1, olen, outf) != olen)
            {
                fprintf (stderr, "%s: write fail of %s: %s\n",
                         prog, output_file, strerror (errno));
                exit (1);
            }
            free (obuf);
            iso2709_rm (rec_input);
	    iso2709_rm (rec_output);
        }
        free (buf);
        no++;
    }
    if (outf)
        fclose (outf);
    return 0;
}
